﻿using System;
using System.Runtime.InteropServices;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using SlimDX;

using DX10 = SlimDX.Direct3D10;
using DXGI = SlimDX.DXGI;
using D3DCompiler = SlimDX.D3DCompiler;

namespace TutorialSeries.DirectX10.Chapter2
{
    public class SimpleTriangle
    {
        private DX10.Buffer m_vertexBuffer;
        private DX10.InputLayout m_vertexLayout;
        private DX10.Effect m_effect;
        private DX10.EffectTechnique m_effectTechnique;
        private DX10.EffectPass m_effectPass;

        public void LoadResources(DX10.Device device)
        {
            Vertex[] vertices = new Vertex[]
            {
                new Vertex(new Vector4(-0.8f, -0.8f, 0f, 1f), Color.Red.ToArgb()),
                new Vertex(new Vector4(0f, 0.8f, 0f, 1f), Color.Green.ToArgb()),
                new Vertex(new Vector4(0.8f, -0.8f, 0f, 1f), Color.Blue.ToArgb())
            };

            DataStream outStream = new DataStream(3 * Marshal.SizeOf(typeof(Vertex)), true, true);
            outStream.WriteRange(vertices);
            outStream.Position = 0;

            DX10.BufferDescription bufferDescription = new DX10.BufferDescription();
            bufferDescription.BindFlags = DX10.BindFlags.VertexBuffer;
            bufferDescription.CpuAccessFlags = DX10.CpuAccessFlags.None;
            bufferDescription.OptionFlags = DX10.ResourceOptionFlags.None;
            bufferDescription.SizeInBytes = 3 * Marshal.SizeOf(typeof(Vertex));
            bufferDescription.Usage = DX10.ResourceUsage.Default;

            m_vertexBuffer = new DX10.Buffer(device, outStream, bufferDescription);
            outStream.Close();

            m_effect = DX10.Effect.FromFile(
                device, 
                "Shaders/SimpleRendering.fx", "fx_4_0", 
                D3DCompiler.ShaderFlags.None, D3DCompiler.EffectFlags.None,
                null,
                null, null);
            m_effectTechnique = m_effect.GetTechniqueByIndex(0);
            m_effectPass = m_effectTechnique.GetPassByIndex(0);

            DX10.InputElement[] inputElements = new DX10.InputElement[]
            {
                new DX10.InputElement("POSITION",0,DXGI.Format.R32G32B32A32_Float,0,0),
                new DX10.InputElement("COLOR",0,DXGI.Format.R8G8B8A8_UNorm,16,0)
            };

            m_vertexLayout = new DX10.InputLayout(device, inputElements, m_effectPass.Description.Signature);
        }

        public void Render(DX10.Device device)
        {
            device.InputAssembler.SetInputLayout(m_vertexLayout);
            device.InputAssembler.SetPrimitiveTopology(DX10.PrimitiveTopology.TriangleList);
            device.InputAssembler.SetVertexBuffers(0, new DX10.VertexBufferBinding(m_vertexBuffer, Marshal.SizeOf(typeof(Vertex)), 0));

            m_effectPass.Apply();
            device.Draw(3, 0);
        }
    }
}
